Index: channels/chan_mobile.c
===================================================================
--- channels/chan_mobile.c	(revision 925)
+++ channels/chan_mobile.c	(working copy)
@@ -1021,7 +1021,12 @@
 
 	ast_debug(3, "*** mbl_read()\n");
 
+	while (ast_mutex_trylock(&pvt->lock)) {
+		CHANNEL_DEADLOCK_AVOIDANCE(ast);
+	}
+	
 	if (!pvt->owner) {
+		ast_mutex_unlock(&pvt->lock);
 		return &ast_null_frame;
 	}
 	memset(&pvt->fr, 0x00, sizeof(struct ast_frame));
@@ -1035,7 +1040,15 @@
 	pvt->fr.data.ptr = pvt->io_buf + AST_FRIENDLY_OFFSET;
 
 	if ((r = read(pvt->sco_socket, pvt->fr.data.ptr, DEVICE_FRAME_SIZE)) == -1) {
-		ast_log(LOG_ERROR, "read error %d\n", errno);
+		if (errno == 104) {
+			ast_log(LOG_WARNING, "read error %d, connection reset by peer, waiting for a new one\n", errno);
+			close(pvt->sco_socket);
+			pvt->sco_socket = -1;
+			ast_channel_set_fd(ast, 0, -1);
+		} else {
+			ast_log(LOG_ERROR, "read error %d\n", errno);
+		}
+		ast_mutex_unlock(&pvt->lock);
 		return &ast_null_frame;
 	}
 
@@ -1045,8 +1058,11 @@
 	if (pvt->do_alignment_detection)
 		do_alignment_detection(pvt, pvt->fr.data.ptr, r);
 
-	return ast_dsp_process(ast, pvt->dsp, &pvt->fr);
+	r = ast_dsp_process(ast, pvt->dsp, &pvt->fr);
+	
+	ast_mutex_unlock(&pvt->lock);
 
+	return r;
 }
 
 static int mbl_write(struct ast_channel *ast, struct ast_frame *frame)
@@ -1061,6 +1077,10 @@
 		return 0;
 	}
 
+	while (ast_mutex_trylock(&pvt->lock)) {
+		CHANNEL_DEADLOCK_AVOIDANCE(ast);
+	}
+	
 	ast_smoother_feed(pvt->smoother, frame);
 
 	while ((f = ast_smoother_read(pvt->smoother))) {
@@ -1068,6 +1088,8 @@
 		ast_frfree(f);
 	}
 
+	ast_mutex_unlock(&pvt->lock);
+	
 	return 0;
 
 }
